"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPayloadTypeFromSchema = exports.getInParamsType = exports.DATABASE_BUILTIN_FIELDS_WITHOUT_ID = exports.DATABASE_BUILTIN_FIELDS = void 0;
const graphql_1 = require("graphql");
const scalar_1 = require("./scalar");
const weda_node_sdk_1 = require("@cloudbase/weda-node-sdk");
const schema_to_type_1 = require("./schema-to-type");
const utils_1 = require("../utils");
/**
 * 自建数据源内置字段
 */
exports.DATABASE_BUILTIN_FIELDS = ['_id', 'createBy', 'createdAt', 'owner', 'updatedAt', 'updateBy'];
// 内置字段中除了 _id 之外的字段
exports.DATABASE_BUILTIN_FIELDS_WITHOUT_ID = exports.DATABASE_BUILTIN_FIELDS.filter(k => k !== '_id');
/**
 * 获取数据源方法入参 args 对象
 * @param datasource 数据源对象
 * @param method 方法名, 字符串表示内置方法名, 对象则为自定义方法名
 * @returns 入参为空时, 返回空, 否则返回字段配置对象
 */
function getInParamsType(datasource, method) {
    if (typeof method === 'string') {
        const payloadTypeName = utils_1.getMethodInTypeName(datasource.name, method, true);
        const standardMethodName = weda_node_sdk_1.normalizeBuiltinMethodName(method);
        const handler = builtinMethodPayloadType[standardMethodName];
        if (!handler) {
            throw new Error(`method ${method} not implemented in graphql mode`);
        }
        return handler(payloadTypeName, datasource);
    }
    if (!method.inParams)
        return;
    return getPayloadTypeFromSchema(utils_1.getMethodInTypeName(datasource.name, method.name, true), method.inParams);
}
exports.getInParamsType = getInParamsType;
/** 内置方法类型 */
const builtinMethodPayloadType = {
    wedaGetItem() {
        return { _id: { type: graphql_1.GraphQLString } };
    },
    wedaGetList() {
        return { params: { type: scalar_1.GraphQLAny } };
    },
    wedaGetRecords() {
        return getPayloadTypeGetRecords();
    },
    wedaDelete() {
        return { _id: { type: graphql_1.GraphQLString } };
    },
    wedaCreate(typeName, datasource) {
        return getPayloadTypeFromSchema(typeName, datasource.schema, exports.DATABASE_BUILTIN_FIELDS);
    },
    wedaUpdate(typeName, datasource) {
        return getPayloadTypeFromSchema(typeName, datasource.schema, exports.DATABASE_BUILTIN_FIELDS_WITHOUT_ID);
    },
};
/**
 * getRecords where Graphql 类型
 */
// eslint-disable-next-line @typescript-eslint/naming-convention
const WhereType = new graphql_1.GraphQLList(new graphql_1.GraphQLInputObjectType({
    name: 'WedaWhereType',
    fields: {
        key: { type: graphql_1.GraphQLString, description: '字段名称' },
        rel: { type: graphql_1.GraphQLString, description: '字段与值之间的关系' },
        val: { type: scalar_1.GraphQLAny, description: '字段值' },
    },
}));
/**
 * getRecords order Graphql 类型
 */
// eslint-disable-next-line @typescript-eslint/naming-convention
const OrderType = new graphql_1.GraphQLEnumType({
    name: 'WedaOrderType',
    values: {
        asc: { value: 'asc', description: '升序' },
        desc: { value: 'desc', description: '降序' },
    },
});
/**
 * getRecords 的 入参类型
 */
function getPayloadTypeGetRecords() {
    return {
        orderBy: { type: graphql_1.GraphQLString, description: '排序字段' },
        orderType: { type: OrderType, description: '排序方式' },
        pageNo: { type: graphql_1.GraphQLInt, description: '页码, 从0开始' },
        pageSize: { type: graphql_1.GraphQLInt, description: '每页大小' },
        where: { type: WhereType, description: '筛选条件' },
    };
}
/**
 * 获取入参类型信息
 * @param schema 数据源入参的 json schema 描述
 * @param isRoot 是否为根结点
 * @param ignoredKeys 忽略的字段
 */
function getPayloadTypeFromSchema(typeName, schema, ignoredKeys) {
    if (schema.type !== 'object') {
        return {
            params: {
                type: schema_to_type_1.getTypeFromSchema(typeName, schema),
            },
        };
    }
    const filter = (key) => {
        if (!ignoredKeys)
            return true;
        if (typeof ignoredKeys === 'function')
            return ignoredKeys(key);
        return !ignoredKeys.includes(key);
    };
    return Object.keys(schema.properties)
        .filter(filter)
        .reduce((acc, cur) => {
        acc[cur] = {
            type: schema_to_type_1.getTypeFromSchema(typeName, schema.properties[cur]),
        };
        return acc;
    }, {});
}
exports.getPayloadTypeFromSchema = getPayloadTypeFromSchema;
