"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDsSchemaType = exports.getGetRecordsResponseType = exports.ID_QL_MAP = exports.getOutParamsType = void 0;
const graphql_1 = require("graphql");
const weda_node_sdk_1 = require("@cloudbase/weda-node-sdk");
const schema_to_type_1 = require("./schema-to-type");
const utils_1 = require("../utils");
/**
 * 获取数据源方法出参的 graphql 类型
 * @param datasource 数据源对象
 * @param method 方法名, 字符串表示内置方法, 对象即内置方法
 */
function getOutParamsType(datasource, method) {
    // 内置方法
    if (typeof method === 'string') {
        const responseTypeName = utils_1.getMethodOutTypeName(datasource.name, method, true);
        const standardMethodName = weda_node_sdk_1.normalizeBuiltinMethodName(method);
        const handler = builtinMethodResponseType[standardMethodName];
        if (!handler) {
            throw new Error(`method ${method} not implemented in graphql mode`);
        }
        return handler(responseTypeName, datasource);
    }
    return schema_to_type_1.getTypeFromSchema(utils_1.getMethodOutTypeName(datasource.name, method.name, true), method.outParams, true);
}
exports.getOutParamsType = getOutParamsType;
/** 内置方法类型 */
const builtinMethodResponseType = {
    wedaGetItem(typeName, datasource) {
        return getDsSchemaType(typeName, datasource);
    },
    wedaGetList(typeName, datasource) {
        return getDsSchemaType(typeName, datasource, true);
    },
    wedaGetRecords(typeName, datasource) {
        return getGetRecordsResponseType(typeName, datasource);
    },
    wedaDelete(typeName) {
        return new graphql_1.GraphQLObjectType({ name: typeName, fields: { deleted: { type: graphql_1.GraphQLInt } } });
    },
    wedaCreate(typeName) {
        return new graphql_1.GraphQLObjectType({ name: typeName, fields: exports.ID_QL_MAP });
    },
    wedaUpdate(typeName) {
        return new graphql_1.GraphQLObjectType({ name: typeName, fields: { updated: { type: graphql_1.GraphQLInt } } });
    },
};
/**
 * 入参类型: 适用于 getItem, delete
 */
exports.ID_QL_MAP = {
    _id: { type: graphql_1.GraphQLString },
};
function getGetRecordsResponseType(typeName, datasource) {
    return new graphql_1.GraphQLObjectType({
        name: `${datasource.name}GetRecordsResponseType`,
        fields: {
            pageNo: { type: graphql_1.GraphQLInt },
            pageSize: { type: graphql_1.GraphQLInt },
            total: { type: graphql_1.GraphQLInt },
            records: { type: getDsSchemaType(typeName, datasource, true) },
        },
    });
}
exports.getGetRecordsResponseType = getGetRecordsResponseType;
/**
 * 缓存的返回类型
 *  主要用于缓存自建数据源字段设置的类型, 以及字段设置的数组类型
 */
const CACHED_SCHEMA = {};
/**
 * 获取自建数据源模型的 graphql 类型信息
 * @param datasource 数据源对象
 * @param isList 是否返回列表类型
 */
function getDsSchemaType(typeName, datasource, isList) {
    const dsName = datasource.name;
    const cacheName = isList ? `${dsName}List` : dsName;
    if (!CACHED_SCHEMA[cacheName]) {
        if (isList) {
            CACHED_SCHEMA[cacheName] = new graphql_1.GraphQLList(getDsSchemaType(typeName, datasource));
        }
        else {
            CACHED_SCHEMA[cacheName] = schema_to_type_1.getTypeFromSchema(typeName, datasource.schema, true);
        }
    }
    return CACHED_SCHEMA[cacheName];
}
exports.getDsSchemaType = getDsSchemaType;
